﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectAiroplane;
/// <summary>
/// Класс отвечающий за прорисовку и перемещение объекта-сущности
/// </summary>
public class DrawningAiroplane
{
    /// <summary>
    /// Класс-сущность
    /// </summary>
    public EntityAiroplane? EntityAiroplane { get; private set; }
    /// <summary>
    /// Ширина окна
    /// </summary>
    private int? _pictureWidth;
    /// <summary>
    /// Высота окна
    /// </summary>
    private int? _pictureHeight;
    /// <summary>
    /// Левая координата прорисовки 
    /// </summary>
    private int? _startPosX;
    /// <summary>
    /// Верхняя кооридната прорисовки 
    /// </summary>
    private int? _startPosY;
    /// <summary>
    /// Ширина прорисовки 
    /// </summary>
    private readonly int _drawningAiroplaneWidth = 160;
    /// <summary>
    /// Высота прорисовки 
    /// </summary>
    private readonly int _drawningaAiroplaneHeight = 80;

    /// <summary>
    /// Инициализация свойств
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес</param>
    /// <param name="bodyColor">Основной цвет</param>
    /// <param name="additionalColor">Дополнительный цвет</param>
    /// <param name="toplivbak">Признак наличия бака</param>
    /// <param name="radar">Признак наличия радара</param>
    
    public void Init(int speed, double weight, Color bodyColor, Color
    additionalColor, bool toplivbak, bool radar)
    {
        EntityAiroplane = new EntityAiroplane();
        EntityAiroplane.Init(speed, weight, bodyColor, additionalColor, toplivbak, radar);
        _pictureWidth = null;
        _pictureHeight = null;
        _startPosX = null;
        _startPosY = null;
    }

    /// <summary>
    /// Установка границ поля
    /// </summary>
    /// <param name="width">Ширина поля</param>
    /// <param name="height">Высота поля</param>
    /// <returns>true - границы заданы, false - проверка не пройдена, нельзя разместить объект в этих размерах</returns>
public bool SetPictureSize(int width, int height)
    {
        // TODO проверка, что объект "влезает" в размеры поля
        // если влезает, сохраняем границы и корректируем позицию объекта,если она была уже установлена
        
        if (width >= _drawningAiroplaneWidth && height >= _drawningaAiroplaneHeight)
        {
            _pictureWidth = width;
            _pictureHeight = height;
            
            if (_startPosX.HasValue && _startPosY.HasValue)
            {
                if (_startPosX + _drawningAiroplaneWidth > _pictureWidth)
                {
                    _startPosX = _pictureWidth - _drawningAiroplaneWidth;
                }

                if (_startPosY + _drawningaAiroplaneHeight > _pictureWidth)
                {
                    _startPosY = _pictureHeight - _drawningaAiroplaneHeight;
                }

            } 
            return true;
        }
        return false;
    }
    /// <summary>
    /// Установка позиции
    /// </summary>
    /// <param name="x">Координата X</param>
    /// <param name="y">Координата Y</param>
    public void SetPosition(int x, int y)
    {
        if (!_pictureHeight.HasValue || !_pictureWidth.HasValue)
        {
            return;
        }
        // TODO если при установке объекта в эти координаты, он будет"выходить" за границы формы
        // то надо изменить координаты, чтобы он оставался в этих границах

        if (x < 0) 
        {
            x = 0; 
        }

        else if (x + _drawningAiroplaneWidth > _pictureWidth.Value) 
        {
            x = _pictureWidth.Value - _drawningAiroplaneWidth; 
        }

        if (y < 0) 
        {
            y = 0; 
        }

        else if (y + _drawningaAiroplaneHeight > _pictureHeight.Value) 
        {
            y = _pictureHeight.Value - _drawningaAiroplaneHeight;
        }

        _startPosX = x;
        _startPosY = y;
    }

    /// <summary>
    /// Изменение направления перемещения
    /// </summary>
    /// <param name="direction">Направление</param>
    /// <returns>true - перемещене выполнено, false - перемещение невозможно</returns>
public bool MoveTransport(DirectionType direction)
    {
        if (EntityAiroplane == null || !_startPosX.HasValue ||
        !_startPosY.HasValue)
        {
            return false;
        }
        switch (direction)
        {
            //влево
            case DirectionType.Left:
                if (_startPosX.Value - EntityAiroplane.Step > 0)
                {
                    _startPosX -= (int)EntityAiroplane.Step;
                }
                return true;
            //вверх
            case DirectionType.Up:
                if (_startPosY.Value - EntityAiroplane.Step > 0)
                {
                    _startPosY -= (int)EntityAiroplane.Step;
                }
                return true;
            // вправо
            case DirectionType.Right:
                if (_startPosX.Value + _drawningAiroplaneWidth + EntityAiroplane.Step < _pictureWidth)
                {
                    _startPosX += (int)EntityAiroplane.Step;
                }
                return true;
            //вниз
            case DirectionType.Down:
                if (_startPosY.Value + _drawningaAiroplaneHeight + EntityAiroplane.Step < _pictureHeight)
                {
                    _startPosY += (int)EntityAiroplane.Step;
                }
                return true;
            default:
                return false;
        }
    }
    /// <summary>
    /// Прорисовка объекта
    /// </summary>
    /// <param name="g"></param>
    public void DrawTransport(Graphics g)
    {
        if (EntityAiroplane == null || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return;
        }

        Pen pen = new(Color.Black);
        Brush additionalBrush = new
        SolidBrush(EntityAiroplane.AdditionalColor);

        

        //////
        //крыло верхнее самолета
        g.DrawLine(pen, _startPosX.Value + 15, _startPosY.Value + 5, _startPosX.Value + 15, _startPosY.Value + 5);
        g.DrawLine(pen, _startPosX.Value + 15, _startPosY.Value + 5, _startPosX.Value + 55, _startPosY.Value + 35);
        g.DrawLine(pen, _startPosX.Value + 15, _startPosY.Value + 35, _startPosX.Value + 15, _startPosY.Value + 5);

        //задняя часть самолета полукруг
        g.DrawArc(pen, _startPosX.Value + 5, _startPosY.Value + 35, 30,30,90,180);

        //сзади
        g.DrawEllipse(pen, _startPosX.Value + 10, _startPosY.Value + 30, 40,10);
        Brush brbr = new SolidBrush(Color.Brown);
        g.FillEllipse(brbr, _startPosX.Value + 10, _startPosY.Value + 30, 40,10);
        //низ самолета
        g.DrawLine(pen, _startPosX.Value + 16, _startPosY.Value + 65, _startPosX.Value + 135, _startPosY.Value + 65);

        // треуголник спереди
        g.DrawLine(pen, _startPosX.Value + 135, _startPosY.Value + 68, _startPosX.Value + 135, _startPosY.Value + 32);
        g.DrawLine(pen, _startPosX.Value + 135, _startPosY.Value + 32, _startPosX.Value + 160, _startPosY.Value + 50);
        g.DrawLine(pen, _startPosX.Value + 135, _startPosY.Value + 68, _startPosX.Value + 160, _startPosY.Value + 50);
        g.DrawLine(pen, _startPosX.Value + 135, _startPosY.Value + 50, _startPosX.Value + 160, _startPosY.Value + 50);

        //вверх от тругольника до треугольника 
        g.DrawLine(pen, _startPosX.Value + 50, _startPosY.Value + 35, _startPosX.Value + 135, _startPosY.Value + 35);

        //крыло
        g.DrawEllipse(pen, _startPosX.Value + 50, _startPosY.Value + 45, 70, 10);
        Brush brbl = new SolidBrush(Color.Black);
        g.FillEllipse(brbl, _startPosX.Value + 50, _startPosY.Value + 45, 70, 10);
        
        //шасси
        g.DrawLine(pen, _startPosX.Value + 35, _startPosY.Value + 65, _startPosX.Value + 35, _startPosY.Value + 75);
        g.DrawLine(pen, _startPosX.Value + 120, _startPosY.Value + 65, _startPosX.Value + 120, _startPosY.Value + 75);
        g.DrawLine(pen, _startPosX.Value + 25, _startPosY.Value + 75, _startPosX.Value + 45, _startPosY.Value + 75);
        g.DrawEllipse(pen, _startPosX.Value + 15, _startPosY.Value + 70, 10, 10);
        g.DrawEllipse(pen, _startPosX.Value + 45, _startPosY.Value + 70, 10, 10);
        g.DrawEllipse(pen, _startPosX.Value + 115, _startPosY.Value + 70, 10, 10);

        //топливный бак
        if (EntityAiroplane.Toplivbak)
        {
            g.FillRectangle(additionalBrush, _startPosX.Value + 65,_startPosY.Value + 20, 30, 15);
        }
       
        //Радар
        if (EntityAiroplane.Radar)
        {
            Brush brGreen = new SolidBrush(Color.LightGreen);
            g.FillEllipse(brGreen, _startPosX.Value + 135, _startPosY.Value + 50, 10, 10);
            g.DrawEllipse(pen, _startPosX.Value + 135, _startPosY.Value + 50, 10, 10);
        }
    }
}

